/* eslint-disable no-useless-escape */
import {httpGET, httpPOST, httpPUT} from '../utils/HTTPUtil';
import {isValidString} from '../utils/Utils';
import {platform} from '../MFPPush';
const applicationServerPublicKey = "BNqEgHEqLOmF_B5rAkjwbFiTr6k5HzthWYul3UmorHOKXeXubI1D0oRw-nMN30ju6KTle2b_q9vNY1LAFtCz9B8";

export default class ChromeFirefoxPushController {
    async registerDevice () {
      const registration = window.pushReg;
      return new Promise((resolve, reject) => {
        registration.pushManager.getSubscription().then(
          function(subscription) {
            if (subscription) {
              registerUsingToken(subscription)
                .then((data) => resolve(data))
                .catch((err) => reject(err));
            } else {
              const applicationServerKey = urlB64ToUint8Array(applicationServerPublicKey);
              registration.pushManager.subscribe({
                userVisibleOnly: true,
                applicationServerKey: applicationServerKey
              }).then(function(subscription) {
                registerUsingToken(subscription)
                  .then((data) => resolve(data))
                  .catch((err) => reject(err));
              }).catch(function(error) {
                if (Notification.permission === 'denied') {
                } else {
                  console.error('Unable to subscribe to push.', error);
                }
                reject(error);
              });
            }
          }).catch(function(e) {
            console.error('Error thrown while subscribing from ' +
            'push messaging.', e);
            reject(e);
          });
      });
    }
}

/*Get subscription details*/
function registerUsingToken(subscription) {
    localStorage.setItem("token", subscription);
    var rawKey = subscription.getKey ? subscription.getKey('p256dh') : '';
    var key = rawKey ? btoa(String.fromCharCode.apply(null, new Uint8Array(rawKey))) : '';
    var rawAuthSecret = subscription.getKey ? subscription.getKey('auth') : '';
    var authSecret = rawAuthSecret ? btoa(String.fromCharCode.apply(null, new Uint8Array(rawAuthSecret))) : '';

    if (!isValidString(subscription.endpoint) || !isValidString(key) || !isValidString(authSecret)) {
        console.error("Error while getting token values");
        return;
    }
    var tokenValue = {
        "endpoint": subscription.endpoint,
        "userPublicKey": key,
        "userAuth": authSecret,
    };
    const WL = window.WL;

    let token = JSON.stringify(tokenValue);
    let deviceId = WL.BrowserManager.getWLUniqueID();
    let userId = localStorage.getItem("userId");
    if(userId) {
      return registerDevice({
        "deviceId": deviceId,
        "token": token,
        "platform": platform,
        "userId": userId
      });
    } else {
      return registerDevice({
        "deviceId": deviceId,
        "token": token,
        "platform": platform
      });
    }
}


function registerDevice(device) {
  // Checking if the existing device registration token is valid.
  return new Promise((resolve, reject) => {
    httpGET('/devices/' + device.deviceId).then((resGET) =>{
        let existingDevice = JSON.parse(resGET.responseText);
        if (existingDevice.token !== device.token || existingDevice.deviceId !== device.deviceId || (device.hasOwnProperty('userId') && (existingDevice.userId !== "anonymous" && existingDevice.userId !== device.userId))) {
          httpPUT('/devices/' + device.deviceId, device, {'Content-Type': 'application/json'})
          .then((resPUT) => { resolve(resPUT); })
          .catch(errPUT => { reject(errPUT); });
        } else {
          resolve();
        }
    }).catch(errGET => {
      if(errGET && errGET.status === 404) {
        if(errGET.responseJson && errGET.responseJson == null) {
          reject("Server Error" + JSON.stringify(errGET));
        } else {
          console.error("Error registering device. " + JSON.stringify(errGET));
          httpPOST('/devices/', device, {'Content-Type': 'application/json'})
            .then((resPOST) => { resolve(resPOST); })
            .catch(errPOST => { reject(errPOST); });
        }
      } else {
        reject("Error registering device. " + JSON.stringify(errGET));
      }
    });
  });
}

function urlB64ToUint8Array(base64String) {
  const padding = '='.repeat((4 - base64String.length % 4) % 4);
  const base64 = (base64String + padding)
    .replace(/\-/g, '+')
    .replace(/_/g, '/');

  const rawData = window.atob(base64);
  const outputArray = new Uint8Array(rawData.length);

  for (let i = 0; i < rawData.length; ++i) {
    outputArray[i] = rawData.charCodeAt(i);
  }
  return outputArray;
}
