/* eslint-disable no-unused-vars */

export default class SafariController {
  async registerDevice() {
    return requestPermissionAndRegisterDevice();
  }
}

/**
 * This method checks for the Notification Permission in respective Browser Platform.
 */
const requestPermissionAndRegisterDevice = function () {
  const WL = window.WL;
  let appId = localStorage.getItem("appId");
  let serverUrl = localStorage.getItem("serverUrl");
  let safariWebsitePushId = localStorage.getItem("safariWebsitePushId");
  if (!safariWebsitePushId) {
    return Promise.reject(
      "Please provide 'safariWebsitePushId' while initializing MFPPush."
    );
  }
  let deviceId = WL.BrowserManager.getWLUniqueID();
  let userId = localStorage.getItem("userId");
  let registrationParams = {
    deviceId,
  };
  registrationParams.userId = userId ? userId : "anonymous";

  return new Promise((resolve, reject) => {
    let resultSafariPermission = window.safari.pushNotification.permission(
      safariWebsitePushId
    );
    if (resultSafariPermission.permission === "default") {
      //User never asked before for permission
      window.safari.pushNotification.requestPermission(
        serverUrl + "imfpush/v1/apps/" + appId,
        safariWebsitePushId,
        registrationParams,
        function (resultRequestPermission) {
          console.log("resultRequestPermission ", resultRequestPermission);
          if (resultRequestPermission.permission === "granted") {
            console.log(
              "The user has granted the permission to receive notifications"
            );
          } else {
            console.log(
              "The user has denied the permission to receive notifications"
            );
          }
          resolve();
        }
      );
    } else if (resultSafariPermission.permission === "denied") {
      console.log("Permission for Notifications was denied");
      resolve();
    } else {
      console.log("Permission already granted");
      resolve();
    }
  });
};
